var ActiveState, LinkMixin, RESERVED_PROPS, React, getUnreservedProps, isLeftClick, isModifiedEvent, makeHref, mergeProperties, transitionTo, withoutProperties;

React = require('react');

ActiveState = require('react-router/modules/mixins/ActiveState');

makeHref = require('react-router/modules/helpers/makeHref');

transitionTo = require('react-router/modules/helpers/transitionTo');

withoutProperties = require('react-router/modules/helpers/withoutProperties');

mergeProperties = require('react-router/modules/helpers/mergeProperties');

RESERVED_PROPS = {
  to: true,
  className: true,
  activeClassName: true,
  query: true,
  children: true
};

LinkMixin = {
  mixins: [ActiveState],
  statics: {
    getUnreservedProps: getUnreservedProps
  },
  propTypes: {
    to: React.PropTypes.string.isRequired,
    query: React.PropTypes.object
  },
  getInitialState: function() {
    return {
      isActive: false
    };
  },
  getParams: function() {
    return getUnreservedProps(this.props, this.additionalReservedProps);
  },
  getHref: function() {
    return makeHref(this.props.to, this.getParams(), this.props.query);
  },
  componentWillReceiveProps: function(nextProps) {
    var params;
    params = getUnreservedProps(nextProps, this.additionalReservedProps);
    return this.setState({
      isActive: ActiveState.statics.isActive(nextProps.to, params, nextProps.query)
    });
  },
  updateActiveState: function() {
    return this.setState({
      isActive: ActiveState.statics.isActive(this.props.to, this.getParams(), this.props.query)
    });
  },
  handleClick: function(event) {
    if (isModifiedEvent(event) || !isLeftClick(event)) {
      return;
    }
    event.preventDefault();
    return transitionTo(this.props.to, this.getParams(), this.props.query);
  }
};

isLeftClick = function(event) {
  return event.button === 0;
};

isModifiedEvent = function(event) {
  return !!(event.metaKey || event.altKey || event.ctrlKey || event.shiftKey);
};

getUnreservedProps = function(props, additionalReservedProps) {
  var reservedProps;
  if (additionalReservedProps) {
    reservedProps = mergeProperties({}, RESERVED_PROPS);
    reservedProps = mergeProperties(reservedProps, additionalReservedProps);
  } else {
    reservedProps = RESERVED_PROPS;
  }
  return withoutProperties(props, reservedProps);
};

module.exports = LinkMixin;
